package org.q.app.config;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.collection.NewCollection;

abstract class AbstractResources implements Resources {

	private static final IniPropertiesConstructor INI_FORMAT = new IniPropertiesConstructor();
	protected final LogTemplate log = new LogTemplate(getClass());
	private final String resourceDir = getResourceDir();

	@Override
	public String loadString(final String path) throws IOException {
		log.debug("Loading resource: %", path);
		return FileUtils.readFileToString(new File(getAbsolutePath(path)));
	}

	@Override
	public Map<String, String> loadIniFile(final String path) throws IOException {
		return INI_FORMAT.constructObject(loadString(path));
	}

	@Override
	public void storeIniFile(final String path, final Map<String, String> properties) throws IOException {
		storeString(path, INI_FORMAT.constructString(properties));
	}

	@Override
	public void storeString(final String path, final String data) throws IOException {
		File file = new File(getAbsolutePath(path));
		log.debug("Saving resource %", file.getPath());
		FileUtils.writeStringToFile(file, data);
	}

	@Override
	public String getAbsolutePath(final String relative) throws IOException {
		return new File(UrlTool.append(resourceDir, relative)).getAbsolutePath();
	}

	@Override
	public List<String> listAll(final String relative) throws IOException {
		return getChildren(resourceDir, relative);
	}

	private List<String> getChildren(final String root, final String path) throws IOException {
		File[] children = new File(UrlTool.append(root, path)).listFiles();

		List<String> resources = NewCollection.list();
		if (children == null) {
			return resources;
		}

		for (File child : children) {
			String nested = UrlTool.append(path, child.getName());
			if (child.isDirectory()) {
				resources.addAll(getChildren(root, nested));
			}
			if (child.isFile()) {
				resources.add(nested);
			}
		}
		Collections.sort(resources);
		return resources;
	}

	abstract String getResourceDir();
}
