package org.q.app.contract;

/**
 * ShellCommands that can be invoked without any swallowed content must implement this interface.
 * 
 * @author Erik Lievaart
 */
public interface Invocable {

	/**
	 * Invoke the ShellCommand.
	 * 
	 * @throws Exception
	 *             any Exception can be thrown.
	 */
	void invoke() throws Exception;

	/**
	 * Validate the ShellCommand. This method is called after all flags have been set and before the command is invoked.
	 * 
	 * @throws ValidationException
	 *             Signals that the current arguments are not valid.
	 */
	void validate() throws ValidationException;

}
