package org.q.app.contract;

/**
 * ShellCommands that can be invoked with swallowed content must implement this interface.
 * 
 * @author Erik Lievaart
 */
public interface Swallows {
	/**
	 * Invoke the ShellCommand.
	 * 
	 * @param raw
	 *            swallowed content.
	 * @throws Exception
	 *             any Exception can be thrown.
	 */
	public void invoke(String raw) throws Exception;

	/**
	 * Validate the ShellCommand. This method is called after all flags have been set and before the command is invoked.
	 * 
	 * @param raw
	 *            swallowed content.
	 * @throws ValidationException
	 *             Signals that the current arguments are not valid.
	 */
	void validate(String raw) throws ValidationException;
}
