package org.q.app.engine.concurrent;

import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import org.q.app.engine.impl.PluginRunnable;
import org.q.toolkit.lang.collection.LazyMap;
import org.q.toolkit.lang.collection.LazyMap.Lookup;
import org.q.toolkit.lang.collection.NewCollection;

class SingleThreadQueue {

	private final Map<String, Future<?>> last = NewCollection.map();
	private final LazyMap<String, ExecutorService> executors = new LazyMap<String, ExecutorService>(
			new Lookup<String, ExecutorService>() {
				@Override
				public ExecutorService find(final String key) {
					return Executors.newSingleThreadExecutor();
				}
			});

	public synchronized void invokeLater(final PluginRunnable runnable) {
		String command = runnable.getCommandName();
		cancelLast(command);

		Future<?> future = executors.get(command).submit(runnable);
		last.put(command, future);
	}

	private void cancelLast(final String command) {
		Future<?> running = last.get(command);
		if (running != null) {
			running.cancel(true);
		}
	}

}
