package org.q.app.engine.context;

import java.io.IOException;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.q.app.config.ApplicationResources;
import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

/**
 * Context class for storing and retrieving variables. In its essence, these are the application configuration settings.
 * 
 * @author Erik Lievaart
 */
public class UserVariables extends AbstractUserContext {

	Map<String, String> defaults;

	@Inject
	UserVariables(final ConfigurableResources classpath, final ApplicationResources application) {
		super(classpath, ResourcePaths.VARIABLE_CONFIG);

		try {
			defaults = application.loadIniFile(ResourcePaths.VARIABLE_CONFIG);
		} catch (IOException e) {
			throw new RuntimeIOException("Unable to read variables.ini", e);
		}
	}

	@Override
	void savePermanentVariables(final Map<String, String> variables) {
		Map<String, String> store = NewCollection.map();

		for (String name : variables.keySet()) {
			if (!StringUtils.equals(variables.get(name), defaults.get(name))) {
				store.put(name, variables.get(name));
			}
		}
		super.savePermanentVariables(store);
	}
}
