package org.q.app.engine.impl;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
class EngineTemplates {

	private final Map<Character, String> templates = NewCollection.concurrentMap();
	private String fallback;

	@Inject
	void init(final ConfigurableResources resources) {
		init(resources.loadIniFile(ResourcePaths.ENGINE_TEMPLATES));
	}

	void init(final Map<String, String> ini) {
		fallback = ini.remove("");

		for (String key : ini.keySet()) {
			String format = "Fatal Error; Failed to load %. Expecting a zero or one non-letter key, but was: %.";
			Check.matches(key, "[^a-zA-Z]?", format, ResourcePaths.ENGINE_TEMPLATES, key);
			templates.put(key.charAt(0), ini.get(key));
		}
	}

	String apply(final String raw) {
		String content = isDefault(raw) ? raw : raw.substring(1);
		return StringUtils.replace(lookup(raw), "${}", content);
	}

	private String lookup(final String raw) {
		return isDefault(raw) ? fallback : templates.get(raw.charAt(0));
	}

	private boolean isDefault(final String raw) {
		return Str.isEmpty(raw) || templates.get(raw.charAt(0)) == null;
	}

}
