package org.q.app.engine.impl;

import java.util.Iterator;
import java.util.Map;

import org.q.app.contract.PluginContext;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.engine.context.BookmarkStore;
import org.q.app.engine.meta.CommandMetadata;
import org.q.app.engine.meta.CommandNameMetadata;
import org.q.app.engine.meta.FlagMetadata;
import org.q.app.engine.meta.MetadataCollection;
import org.q.app.engine.parse.ShellArgument;
import org.q.app.engine.parse.ShellCommand;
import org.q.app.engine.parse.ShellParseException;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.test.AntProperties;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
class PluginMetadata {

	@Inject
	private MetadataCollection metadata;
	@Inject
	private BookmarkStore bookmarks;

	void validate(final ShellCommand command) throws ValidationException, ShellParseException {
		metadata.validate(command);
	}

	CommandNameMetadata getName(final ShellCommand command) {
		return metadata.lookup(command.getName()).getName();
	}

	Map<String, String[]> loadFlags(final ShellCommand command, final PluginContext context) {
		CommandMetadata info = metadata.lookup(command.getName());

		Iterator<ShellArgument> iter = command.getArguments();
		Map<String, String[]> flagArguments = NewCollection.map();

		for (char c : info.getFlagsOrDefaults(command)) {
			FlagMetadata flag = info.getFlagMetadata(c);
			flagArguments.put(flag.getName(), extractArguments(flag.getShellArguments(iter), context));
		}
		return flagArguments;
	}

	String[] extractArguments(final ShellArgument[] arguments, final PluginContext context) {

		String[] parameters = new String[arguments.length];
		for (int i = 0; i < arguments.length; i++) {
			parameters[i] = resolve(arguments[i], context);
		}
		return parameters;
	}

	String resolve(final ShellArgument argument, final PluginContext context) {
		switch (argument.getType()) {

		case BOOKMARK:
			String bookmark = bookmarks.getVariable(argument.getValue());
			Check.notNull(bookmark, "Bookmark not found: %", argument.getValue());
			return bookmark;

		case VARIABLE:
			String variable = context.getVariables().get(argument.getValue());
			Check.notNull(variable, "Variable not found: %", argument.getValue());
			return variable;

		case STRING:
			return argument.getValue();
		}
		throw new IllegalArgumentException(Str.sub("Unknown enum constant: %", argument.getType()));
	}

	ThreadPolicy getThreadPolicy(final ShellCommand command) {
		if (AntProperties.isTestMode()) {
			return ThreadPolicy.CURRENT;
		}
		return metadata.lookup(command.getName()).getThreadPolicy();
	}

}
