package org.q.app.engine.meta;

import java.util.List;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.CollectionCheck;
import org.q.toolkit.lang.concurrent.Immutable;

@Immutable
class FlagGroupMetadata {

	private final List<FlagMetadata> flags;
	private final FlagMetadata primary;
	private final String group;

	public FlagGroupMetadata(final String group, final List<FlagMetadata> flags, final FlagMetadata primary) {
		this.group = group;
		this.flags = flags;
		this.primary = primary;

		checkAssertionExceptions();
	}

	private void checkAssertionExceptions() {
		Check.notNull(group);
		if (primary != null) {
			CollectionCheck.isPresent(flags, primary, "Group % does not contain primary flag %", group, primary);
		}
	}

	List<FlagMetadata> getAllFlags() {
		return flags;
	}

	boolean containsFlag(final char c) {
		for (FlagMetadata item : flags) {
			if (item.getFlag() == c) {
				return true;
			}
		}
		return false;
	}

	public String getName() {
		return group;
	}

	boolean hasPrimaryFlag() {
		return primary != null;
	}

	Character getPrimaryFlag() {
		return primary.getFlag();
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append(group).append("primary", primary).toString();
	}

}
