package org.q.app.engine.script;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.WeakHashMap;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.q.app.config.ScriptResources;
import org.q.app.config.VariableConstants;
import org.q.app.engine.context.UserVariables;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.collection.LazyMap;
import org.q.toolkit.lang.collection.LazyMap.Lookup;

import com.google.inject.Inject;

/**
 * Caches scripts if the "q.scripts.cache" variable is set to true. This will reduce IO and give the application a
 * marginal performance improvement.
 * 
 * @author Erik Lievaart
 */
public class ScriptCache {

	@Inject
	private UserVariables variables;
	@Inject
	private ScriptResources resources;

	private LazyMap<String, String> cache = new LazyMap<String, String>(lookup(), new WeakHashMap<String, String>());

	String readResource(final String resource) throws IOException {
		if (StringUtils.equalsIgnoreCase(variables.getVariable(VariableConstants.SCRIPT_CACHE), "true")) {
			cache.get(resource);
		}
		return load(resource);
	}

	private String load(final String resource) throws IOException, FileNotFoundException {
		return IOUtils.toString(new FileInputStream(resources.getAbsolutePath(resource)));
	}

	private Lookup<String, String> lookup() {
		return new Lookup<String, String>() {
			@Override
			public String find(final String key) {
				try {
					return load(key);
				} catch (IOException e) {
					throw new RuntimeIOException(e);
				}
			}
		};
	}
}
