package org.q.app.general;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;

/**
 * This easy to use class implements overrides of the equals, hashcode and toString methods based on the EqualsBuilder,
 * HashCodeBuilder and ToStringBuilder of apache lang using reflection. This is a genuine, Eazy Erik approved, hack. Why
 * is it a hack you ask? It abuses the inheritance hierarchy for a completely different purpose than it was intended
 * for. Therefore, do NOT use this class in instanceof tests. Classes that extend ObjectOverrides do not share any
 * common characteristic, so from a pure OO perspective this class is an abomination. Do not extend this class with
 * utilities, because this ties the extending class to a class it should not be aware of. On the bright side though, it
 * reduces clutter by not requiring every class to add these three simple overrides. Its main purpose is supporting
 * debugging & unit tests. The toString method is useful for debugging. Many classes don't require a hashcode and equals
 * method. This makes these classes hard to unit test, though. I don't like the idea of cluttering production code for
 * testing purposes and this parent class restricts the clutter to a minimum.
 * 
 * @author Erik Lievaart
 */
public abstract class ObjectOverrides {

	@Override
	public boolean equals(final Object obj) {
		return EqualsBuilder.reflectionEquals(this, obj);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}
}
