package org.q.app.ui;

import java.util.Map;

import org.q.app.contract.PluginContext;
import org.q.app.ui.context.ContextMediator;

import com.google.inject.Inject;
import com.google.inject.Provider;

/**
 * Guice {@link Provider} for the PluginContext. The {@link PluginContext} in part contains a snapshot of file
 * selections and such. Injecting the {@link PluginContext} itself would inject a snapshot of the opened locations and
 * file selections at the time the Guice component was created, this is possibly not what you want. Inject the
 * {@link Provider} instead, so that the {@link PluginContext} retrieved will contain a snapshot at the time of
 * invocation.
 * 
 * @author Erik Lievaart
 */
class PluginContextProvider implements Provider<PluginContext> {

	@Inject
	private ContextMediator mediator;

	private PluginContextProvider() {
	}

	@Override
	public PluginContext get() {
		return new PluginContextImpl(mediator.getEngineVariables());
	}

	private class PluginContextImpl implements PluginContext {
		private final Map<String, String> context;

		public PluginContextImpl(final Map<String, String> context) {
			this.context = context;
		}

		@Override
		public Map<String, String> getVariables() {
			return context;
		}
	}
}
