package org.q.app.ui.bind;

import java.awt.Component;
import java.awt.Window;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.JList;
import javax.swing.event.ListSelectionListener;

import org.q.toolkit.lang.Check;
import org.q.toolkit.reflect.InstanceTool;

enum ConfigureListener {

	$WINDOW_CLOSING {
		@Override
		void on(final Component component, final Action action) {
			Check.isInstance(Window.class, component);
			InstanceTool.getMethodWrapper("addWindowListener", component).invoke(WrapAction.windowClosing(action));
		}
	},
	$ACTION {
		@Override
		void on(final Component component, final Action action) {
			InstanceTool.getMethodWrapper("setAction", component).invoke(action);
		}
	},
	$FOCUS_LOST {
		@Override
		void on(final Component component, final Action action) {
			component.addFocusListener(WrapAction.focusLost(action));
		}
	},
	$FOCUS_GAINED {
		@Override
		void on(final Component component, final Action action) {
			component.addFocusListener(WrapAction.focusGained(action));
		}
	},
	$CLICK1 {
		@Override
		void on(final Component component, final Action action) {
			component.addMouseListener(WrapAction.click(action, MouseEvent.BUTTON1, 1));
		}
	},
	$CLICK2 {
		@Override
		void on(final Component component, final Action action) {
			component.addMouseListener(WrapAction.click(action, MouseEvent.BUTTON2, 1));
		}
	},
	$CLICK3 {
		@Override
		void on(final Component component, final Action action) {
			component.addMouseListener(WrapAction.click(action, MouseEvent.BUTTON3, 1));
		}
	},
	$CLICK1_TWICE {
		@Override
		void on(final Component component, final Action action) {
			component.addMouseListener(WrapAction.click(action, MouseEvent.BUTTON1, 2));
		}
	},
	$LIST_SELECTION {
		@Override
		void on(final Component component, final Action action) {
			Check.isInstance(JList.class, component);
			JList list = (JList) component;
			list.addListSelectionListener(WrapAction.proxy(action, ListSelectionListener.class));
		}
	},
	$KEY_RELEASED {
		@Override
		void on(final Component component, final Action action) {
			component.addKeyListener(WrapAction.keyReleased(action));
		}
	};

	abstract void on(Component component, Action action);

}
