package org.q.app.ui.bind;

import javax.swing.JMenuBar;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.app.ui.bind.menu.JMenuBarBuilder;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.Str;
import org.q.toolkit.xml.SaxReflectionReader;

import com.google.inject.Inject;
import com.google.inject.Provider;

/**
 * Guice {@link Provider} for the {@link JMenuBar}
 * 
 * @author Erik Lievaart
 */
public class JMenuBarProvider implements Provider<JMenuBar> {

	private static final String PROCESSING_PREFIX = "org.q.SaxReflectionHandler";

	@Inject
	private ConfigurableResources resource;
	@Inject
	private MenuConvertersProvider converters;

	@Override
	public JMenuBar get() {
		try {
			SaxReflectionReader reader = new SaxReflectionReader(PROCESSING_PREFIX, converters.get());
			JMenuBarBuilder factory = reader.readXmlFile(resource.getAbsolutePath(ResourcePaths.MENU_BAR));
			return factory.instance();
		} catch (Exception e) {
			throw new RuntimeIOException(Str.sub("Unable to read resource %", ResourcePaths.MENU_BAR), e);
		}
	}

}
