package org.q.app.ui.bind.menu;

import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.Str;
import org.q.toolkit.reflect.FieldTool;

/**
 * Factory class for building a {@link JMenuItem}. This class should be used as a prototype.
 * 
 * @author Erik Lievaart
 */
@Contract
public class JMenuItemBuilder implements ComponentBuilder<JMenuItem> {

	private final JMenuItem instance = new JMenuItem();
	private KeyStroke accelerator;
	private String text;

	/**
	 * Set the label of the {@link JMenuItem}.
	 */
	public void setText(final String text) {
		this.text = text;
	}

	/**
	 * Set the mnemonic of the {@link JMenuItem}.
	 * 
	 * @param text
	 *            The name of a constant on the KeyEvent class.
	 */
	public void setMnemonic(final String text) {
		instance.setMnemonic((Integer) FieldTool.getConstant(KeyEvent.class, text));
	}

	/**
	 * Set the accelerator for the JMenuItem.
	 */
	public void setAccelerator(final KeyStroke accelerator) {
		this.accelerator = accelerator;
	}

	/**
	 * Set the action to invoke when the {@link JMenuItem} is selected.
	 */
	public void setAction(final Action action) {
		instance.setAction(action);
	}

	@Override
	public void addChild(final ComponentBuilder<?> factory) {
		throw new UnsupportedOperationException(Str.sub("JMenuItem % does not take children", instance.getText()));
	}

	@Override
	public JMenuItem instance() {
		// Acellerator and text have to be registered AFTER the action is registered.
		instance.setText(text);
		instance.setAccelerator(accelerator);
		return instance;
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}
}
