package org.q.app.ui.context;

import java.util.List;
import java.util.Map;

import org.q.app.engine.context.EngineSnapshot;
import org.q.app.engine.context.FileSnapshot;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.collection.NewCollection;

class EngineVariables {

	private EngineVariables() {
	}

	static Map<String, String> resolve(final EngineSnapshot snapshot) {
		Map<String, String> map = NewCollection.map();
		if (snapshot.getLeft().getDir() == null || snapshot.getRight().getDir() == null) {
			return map; // on startup
		}

		addFileSnapshot(map, snapshot.getActive(), "");
		addFileSnapshot(map, snapshot.getInactive(), "~");
		addFileSnapshot(map, snapshot.getLeft(), "1");
		addFileSnapshot(map, snapshot.getRight(), "2");
		return map;
	}

	private static void addFileSnapshot(final Map<String, String> map, final FileSnapshot files, final String suffix) {
		addSingle(map, files, suffix);
		addMulti(map, files, suffix);
	}

	private static void addSingle(final Map<String, String> map, final FileSnapshot snapshot, final String suffix) {
		map.put("dir" + suffix, snapshot.getDir().getUrlEscaped());
		map.put("name" + suffix, snapshot.getFile().getName());
		map.put("url" + suffix, snapshot.getFile().getUrlEscaped());
		if (snapshot.getDir().getParentFile() != null) {
			map.put("parent" + suffix, snapshot.getDir().getParentFile().getUrlEscaped());
		}
	}

	private static void addMulti(final Map<String, String> map, final FileSnapshot snapshot, final String suffix) {
		List<VirtualFile> files = snapshot.getFiles();
		StringBuilder filesBuilder = new StringBuilder(snapshot.getFile().getUrlEscaped());
		StringBuilder namesBuilder = new StringBuilder(UrlTool.escape(snapshot.getFile().getName()));
		for (int i = 1; i < files.size(); i++) {
			filesBuilder.append(" ").append(files.get(i).getUrlEscaped());
			namesBuilder.append(" ").append(UrlTool.escape(files.get(i).getName()));
		}
		map.put("urls" + suffix, filesBuilder.toString());
		map.put("names" + suffix, namesBuilder.toString());
	}

}
