package org.q.app.ui.render.browser;

import java.io.File;
import java.io.IOException;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;
import javax.swing.plaf.metal.MetalIconFactory;

import org.q.app.ui.render.list.IconFactory;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.RaceCondition;


@RaceCondition("Only use this class from the Swing event thread.")
class OsIconFactory implements IconFactory {
	private static final boolean LINUX = System.getProperty("os.name").equalsIgnoreCase("linux");
	private static final Icon FILE_ICON = MetalIconFactory.getTreeLeafIcon();
	private static final Icon DIR_ICON = MetalIconFactory.getTreeFolderIcon();

	private Icon directory;
	private static final LogTemplate log = new LogTemplate(OsIconFactory.class);
	private static final Map<String, Icon> CACHE = NewCollection.weakMap();

	static {
		if (LINUX) {
			log.info("Native Icons unavailable under Linux, returning default icons.");
		}
	}

	@Override
	public Icon getIcon(final Object file) {
		return getIcon((VirtualFileWrapper) file);
	}

	public Icon getIcon(final VirtualFileWrapper file) {
		return file.getVirtualFile().isDirectory() ? getDirectoryIcon(file) : getSystemIcon(file);
	}

	private Icon getSystemIcon(final VirtualFileWrapper file) {
		String ext = file.getVirtualFile().getExtension();
		if (!CACHE.containsKey(ext) || CACHE.get(ext) == null) {
			try {
				CACHE.put(ext, loadSystemIcon(file));
			} catch (IOException e) {
				log.warn("Unable to read system icon for: " + file.getVirtualFile(), e);
				return null;
			}
		}
		return CACHE.get(ext);
	}

	private Icon getDirectoryIcon(final VirtualFileWrapper file) {
		if (directory == null) {
			directory = loadFolderIcon(file);
		}
		return directory;
	}

	private Icon loadFolderIcon(final VirtualFileWrapper wrapper) {
		if (LINUX) {
			return DIR_ICON;
		}
		if (!wrapper.getVirtualFile().isLocal()) {
			return null;
		}
		File file = wrapper.getVirtualFile().getLocalFile();
		return FileSystemView.getFileSystemView().getSystemIcon(file);
	}

	private Icon loadSystemIcon(final VirtualFileWrapper wrapper) throws IOException {
		if (LINUX) {
			return FILE_ICON;
		}
		String ext = "." + wrapper.getVirtualFile().getExtension();
		File file = File.createTempFile("icon" + System.currentTimeMillis(), ext);
		Icon icon = FileSystemView.getFileSystemView().getSystemIcon(file);
		file.delete();
		return icon;
	}
}
