package org.q.plugins.delete;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import org.q.app.vfs.VirtualFile;

import com.google.inject.Singleton;

@Singleton
class TrashCache {

	static final String TRASH_NAME = ".Trash-1000";
	static final String FILES_NAME = "files";

	private final Set<VirtualFile> trashParents = new HashSet<VirtualFile>();
	private final Map<String, Boolean> unavailable = new WeakHashMap<String, Boolean>();

	VirtualFile getTrashLocation(final VirtualFile commons) {
		if (!commons.isLocal()) {
			return null; // Don't use trash for remote locations, finding the trash might add too much overhead
		}

		String deleteUrl = commons.getUrlEscaped();
		for (VirtualFile parent : trashParents) {
			if (deleteUrl.startsWith(parent.getUrl())) {
				return getTrashFolder(parent);
			}
		}
		return lookup(commons.getParentFile());
	}

	private VirtualFile lookup(final VirtualFile commons) {
		String url = commons.getUrlEscaped();
		if (unavailable.containsKey(url)) {
			return null;
		}
		if (commons.resolveFile(TRASH_NAME).exists()) {
			VirtualFile trash = getTrashFolder(commons);
			trashParents.add(trash);
			return trash;
		}
		unavailable.put(url, true);
		return commons.hasParent() ? lookup(commons.getParentFile()) : null;
	}

	private VirtualFile getTrashFolder(final VirtualFile parent) {
		return parent.resolveFile(TRASH_NAME).resolveFile(FILES_NAME);
	}

}
