package org.q.plugins.replace;

import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.q.app.contract.PluginContext;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.CollectionCheck;
import org.q.toolkit.lang.collection.LazyMap;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

class TokenReplacer {
	private static final Pattern TOKEN_PATTERN = Pattern.compile("\\$\\{([^}]++)\\}");

	private boolean path = false;
	private boolean url = false;
	private String separator = " ";
	private String replace = "";
	private String with = "";

	Map<String, String> variables;

	@Inject
	void init(final PluginContext context) {
		this.variables = context.getVariables();
		CollectionCheck.notEmpty(variables, "No variables: $", context.getClass());
	}

	void replaceWhitespace(final String separator) {
		this.separator = separator;
	}

	void replaceUrlCharacters() {
		this.url = true;
	}

	void removeProtocol() {
		this.path = true;
	}

	void escapeSubstring(final String from, final String to) {
		this.replace = from;
		this.with = to;
	}

	String getPath(final String raw) {
		return path ? UrlTool.getPath(raw) : raw;
	}

	String escapeUrl(final String raw) {
		return url ? raw : UrlTool.unescape(raw);
	}

	String replaceSubstring(final String raw) {
		return StringUtils.replace(raw, replace, with);
	}

	String escapeWhitespace(final List<String> raw) {
		return StringUtils.join(raw, separator);
	}

	String convert(final String raw) {
		return replaceSubstring(escapeUrl(getPath(raw)));
	}

	Map<String, String> createReplacements() {
		return new LazyMap<String, String>(new LazyMap.Lookup<String, String>() {
			@Override
			public String find(final String key) {
				List<String> converted = NewCollection.list();

				String variable = variables.get(key.toString());
				Check.notNull(variable, "% not set", key);
				for (String raw : variable.split("\\s")) {
					converted.add(convert(raw));
				}
				return escapeWhitespace(converted);
			}

		});
	}

	String replace(final String raw) {
		return replace(raw, createReplacements());
	}

	/**
	 * @see http://stackoverflow.com/questions/959731/how-to-replace-a-set-of-tokens-in-a-java-string
	 */
	static String replace(final String raw, final Map<String, String> replacements) {
		Matcher matcher = TOKEN_PATTERN.matcher(raw);

		StringBuffer buffer = new StringBuffer();
		while (matcher.find()) {
			String replacement = replacements.get(matcher.group(1));
			if (replacement != null) {
				matcher.appendReplacement(buffer, replacement.replaceAll("\\\\", "\\\\\\\\"));
			}
		}
		matcher.appendTail(buffer);
		return buffer.toString();
	}

}
