package org.q.toolkit.convert.construct;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

import org.q.toolkit.convert.Constructor;
import org.q.toolkit.convert.Converter;
import org.q.toolkit.convert.Validator;

/**
 * Abstract base class for creating {@link Constructor}'s.
 * 
 * @param <T>
 *            generic type of the instances to construct.
 * 
 * @author Erik Lievaart
 */
public abstract class AbstractConstructor<T> implements Constructor<T> {

	/** Type of the instances constructed by this Constructor. */
	protected final Class<T> literal;

	@SuppressWarnings("unchecked")
	public AbstractConstructor() {
		Type type = getClass().getGenericSuperclass();
		literal = (Class<T>) ((ParameterizedType) type).getActualTypeArguments()[0];
	}

	@Override
	public Class<T> getLiteral() {
		return literal;
	}

	@Override
	public Converter<T> createConverter() {
		return new Converter<T>(this);
	}

	@Override
	public Converter<T> createConverter(final Validator validator) {
		return new Converter<T>(this, validator);
	}

}
