package org.q.toolkit.lang;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

public class CollectionCheck {

	private CollectionCheck() {
	}

	public static void notEmpty(final Map<?, ?> map) {
		notEmpty(map, "The Map was empty.");
	}

	public static void notEmpty(final Map<?, ?> map, final String format, final Object... args) {
		Check.notNull(map, format, args);
		Check.isTrue(map.size() > 0, format, args);
	}

	public static <E> void notEmpty(final E[] array) {
		Check.notNull(array);
		Check.isTrue(array.length > 0, "Array does not contain any elements: " + array);
	}

	public static void isEmpty(final Collection<?> match) {
		isEmpty(match, "Collection not empty %", match);
	}

	public static void isEmpty(final Collection<?> match, final String format, final Object... args) {
		Check.isTrue(match.isEmpty(), format, args);
	}

	public static void isSize(final Collection<?> c, final int size) {
		Check.notNull(c);
		Check.isTrue(c.size() == size, "Collection has % expected: % elements", c.size(), size);
	}

	public static void isSize(final Map<?, ?> map, final int size) {
		Check.notNull(map);
		Check.isTrue(map.size() == size, "Map has % elements expected: %", map.size(), size);
	}

	public static <E> void isPresent(final Collection<E> collection, final E item) {
		isPresent(collection, item, "% not in %", item, collection);
	}

	public static <E> void isPresent(final Collection<E> collection, final E item, final String format,
			final Object... args) {
		Check.isTrue(collection.contains(item), format, args);
	}

	public static <E> void notPresent(final Collection<E> collection, final E item, final String format,
			final Object... args) {
		Check.isFalse(collection.contains(item), format, args);
	}

	public static <E, F> void isPresent(final Map<E, F> map, final E key) {
		isPresent(map, key, "% not in Map: %", key, map.keySet());
	}

	public static <E, F> void isPresent(final Map<E, F> map, final E key, final String format, final Object... args) {
		Check.isTrue(map.containsKey(key), format, args);
	}

	public static void notEmpty(final Collection<?> data) {
		Check.notNull(data);
		Check.isTrue(data.size() > 0, "Collection is empty");
	}

	public static void notEmpty(final Collection<?> data, final String format, final Object... args) {
		Check.notNull(data);
		Check.isTrue(data.size() > 0, format, args);
	}

	/**
	 * Compares the content of two iterators and asserts that they are equal (uses up the iterator).
	 */
	public static <E> void isEqual(final Iterator<E> actual, final Iterator<E> expected) {
		if (expected == null) {
			Check.isNull(actual, "Expected <null>, but was: %", actual);
			return;
		}
		while (expected.hasNext()) {
			E next = expected.next();
			Check.isTrue(actual.hasNext(), "Actual is empty, expected: %", next);
			Check.isEqual(actual.next(), next);
		}
		if (expected.hasNext()) {
			throw new AssertionException(Str.sub("Actual has more elements than expected: %", actual.next()));
		}
	}

}
