package org.q.toolkit.lang.collection;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeSet;

/**
 * Utility class for working with Sets.
 * 
 * @author Erik Lievaart
 */
public class SetTool {
	private SetTool() {
	}

	/**
	 * Create an unmodifiable set containing all the unique entries in the original.
	 */
	public static <E> Set<E> unmodifiableCopy(final Collection<E> original) {
		return Collections.unmodifiableSet(new HashSet<E>(original));
	}

	/**
	 * Create a TreeSet from the specified elements.
	 */
	public static <E> TreeSet<E> treeSet(final E... elements) {
		TreeSet<E> set = NewCollection.treeSet();
		for (E element : elements) {
			set.add(element);
		}
		return set;
	}

	/**
	 * Create a Set from the specified elements.
	 */
	public static <E> Set<E> of(final E... elements) {
		Set<E> set = NewCollection.set();
		set.addAll(Arrays.asList(elements));
		return set;
	}

	/**
	 * Merge the supplied Set's into a single Set containing all unique entries.
	 */
	public static <E> Set<E> merge(final Set<E>... sets) {
		Set<E> result = NewCollection.set();
		for (Set<E> set : sets) {
			result.addAll(set);
		}
		return result;
	}
}
